#本程序有三个函数组成，分别为ebt(x0,x,y0,y)、ebt.power(x,p1,y,p2,alpha)和ebt.size(p1,p2,power)
#ebt进行率比较，原假设为两个率相等，输出概率值
#ebt.power根据两组样本量和各自的阳性率估计值以及预设的显著性水平进行统计功效的计算
#ebt.size根据两组各自阳性率的估计值和期望达到的最小统计功效，估计需要的最小总样本量（两组样本量之和）

#ebt进行率的比较
#例：ebt(10,100,50，200)
ebt <- function(x0,x,y0,y){
   #初始化变量x0, x, y0和y
   #x0和x表示group1的阳性数和总数
   #y0和y表示group2的阳性数和总数
   l1 <- length(x0)
   l2 <- length(x)
   l3 <- length(y0)
   l4 <- length(y)
   p <- numeric(l1)
   if (l1 != l2) {
     print("Warnings: Vectors with different length!\n")
     #检测向量长度是否相同
   } else if (l3 != l4) {
     print("Warnings: Vectors with different length!\n")
     #检测向量长度是否相同
   }  else if (l1 != l3) {
     print("Warnings: Vectors with different length!\n")
     #检测向量长度是否相同
   }
   for (i in seq(from=1,to=l1,by=1)){
      #根据率大小进行有向性检验
      if((x0[i]/x[i] - y0[i]/y[i])>0){
        pL <- pbinom((x0[i]-1),x[i],(x0[i]+y0[i])/(x[i]+y[i]))
        pR <- 1 - pbinom(y0[i],y[i],(x0[i]+y0[i])/(x[i]+y[i]))
        #转化至两个二项分布
        p0 <- 1- pL * pR
        #求解联合分布概率
      } else {
        pL <- 1 - pbinom(x0[i],x[i],(x0[i]+y0[i])/(x[i]+y[i]))
        pR <- pbinom((y0[i]-1),y[i],(x0[i]+y0[i])/(x[i]+y[i]))
        #转化至两个二项分布
        p0 <- 1 - pL * pR
        #求解联合分布概率
      }
      p[i] <- p0
   }
   p
   #返回率比较的p值向量
}   


#Power计算
#例：ebt.power(100,0.1,300,0.2,0.05)
ebt.power <- function(x, p1, y, p2, alpha){
   #初始化变量x, p1, y, p2和alpha
   #x表示group1的总样本量
   #p1表示group1的估计阳性率
   #y表示group2的总样本量
   #p2表示group2的估计阳性率
   #alpha表示预设显著性水平，通常取0.05
   if (x <= 0) {
     print("Warnings: The sample number should be a positive integer!\n")
   } 
   else if (y <= 0) {
     print("Warnings: The sample number should be a positive integer!\n")
   }
   else if (p1 < 0 || p1 > 1 || p2 < 0 || p2 > 1) {
     print("Warnings: The estimated occurrence should be a located within 0 ~ 1\n")
   } 
   else if (alpha < 0 || alpha > 1) {
     print("Warnings: Significance level should be a value between 0 and 1\n")
   }
   psum = 0
   for (i in seq(from=0,to=x,by=1)){
      for (j in seq(from=0,to=y,by=1)){
           p <- ebt(i,x,j,y)
           if (p < alpha) {
             if (i==0){
               pa = pbinom(i,x,p1)
             } else {
               pa = pbinom(i,x,p1) - pbinom(i-1,x,p1)
             }
             if (j==0){
               pb = pbinom(j,y,p2)
             } else {
               pb = pbinom(j,y,p2) - pbinom(j-1,y,p2)
             }
             psum = pa * pb + psum
           }
      }
   }
   if (psum < 1){
      psum
   }
   else{
      1
   }
   #统计power的最大值为1
   #输出power
}


#样本量估计
#例：ebt.size(0.1,0.3,0.75)
ebt.size <- function(p1, p2, power){
   #初始化变量p1, p2和power
   #p1表示group1的估计阳性率
   #p2表示group2的估计阳性率
   #power表示期望达到的最小统计功效
   #默认的显著性水平为0.05
   if (p1 < 0 || p1 > 1 || p2 < 0 || p2 > 1) {
     print("Warnings: The estimated occurrence should be a located within 0 ~ 1\n")
   } 
   else if (power < 0 || power > 1) {
     print("Warnings: Power should be a value between 0 and 1\n")
   }
   flag = 0
   for (i in seq(from=10,to=1000,by=10)){
       pow = ebt.power(i,p1,i,p2,0.05)
       if (pow >= power){
         sampleSize = 2*i
         flag = 1
         break         
       }
   }
   if(flag){
     sampleSize
   } else {
     print("Sample size should be larger than 2000\n")
   }
   #样本量的最大值设为2000
   #输出最小总样本量（两组样本量之和） 
}
